from OFS.Folder import Folder
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from AccessControl.User import User, super, nobody
from AccessControl.PermissionRole import _what_not_even_god_should_do
from Globals import HTMLFile, MessageDialog
_marker = []


# XXX need to add __ac_permissions__ on everything

_LoggingInUser = User('LoggingInUser', '', ['LoggingIn'], [])

_LoginMethods = []
_UserSources = []

# Default DTML Documents

defLoginForm = """<dtml-raise "'LoginRequired'"><html>
<head><title>Please log in.</title></head>
<body bgcolor="ffffff">
<br>

<dtml-comment>
  This login form is appropriate for use with the Basic Cookie Auth
  LoginMethod.  Edit it to taste, and to match the cookie names you
  specified, if different than the default. 
  
  If you're only using REMOTE_USER or Basic Auth, you
  don't need a loginForm and can delete this.  If you've developed a
  custom LoginMethod, you may need additional customization.
</dtml-comment>

<br>
<center>

<form method="post" action="<dtml-var SCRIPT_NAME><dtml-var PATH_INFO><dtml-if QUERY_STRING>?<dtml-var QUERY_STRING></dtml-if>">
<table>
 <tr><td></td><td><h1>You must login to the system</h1></td></tr>
 <tr><td></td></tr>
 <tr>
  <th align="right">Login:</th>
  <td><input type="string" name="__ac_name" size=40></td>
 </tr><tr>
  <th align="right">Password:</th>
  <td><input type="password" name="__ac_password" size=40></td>
 </tr><tr>
  <td></td>
  <td><input type="submit" name=" Log In " value=" Log In ">
 </tr>
</table>
</form>

</center>

</body></html>
</dtml-raise>"""

defLogoutForm = """<html><head><title>You have been logged out.</title></head>
<body bgcolor="ffffff">
 <br>
 <center>
  <H1>You have been logged out.</H1>
  <dtml-call "AUTHENTICATED_USER.logout()">
 </center>
</body></html>
"""

defForbiddenPage = """<dtml-raise "'Forbidden'"><dtml-var standard_html_header>
Sorry <dtml-var user>, you are forbidden to access this resource.

<dtml-comment>
  This page, if it exists, is called in the event that the user has been
  validated, but doesn't have rights to perform the requested action. 
 
  If instead you want to have this condition cause the user to be prompted
  to log in again (normal User Folder behavior), you can delete this object.
</dtml-comment>

<dtml-comment>
<!--
  You have these roles: <dtml-var "_.string.join(userroles, ', ')"><br>
  You need one of these roles: <dtml-var "_.string.join(needroles, ', ')"><br>
-->
</dtml-comment>

<dtml-var standard_html_footer>
</dtml-raise>"""
       
manage_addLoginManagerForm = HTMLFile('addLoginManager', globals(),
    UserSourcesMetaTypes = _UserSources,
    LoginMethodsMetaTypes = _LoginMethods)

def manage_addLoginManager(self,
    usource=None, lmeths=[], loginf=1, logoutf=1, forbidp=1,
    REQUEST=None):
    
    """Add a LoginManager to the self folder"""
    
    ob = LoginManager()

    try: self._setObject('acl_users', ob)
    except: return MessageDialog(
                    title  ='Item Exists',
                    message='This object already contains an acl_users',
                    action ='%s/manage_main' % REQUEST['PARENT_URL'])
                                                              
    self.__allow_groups__ = ob

    # Verify that it's a valid method!
    if usource in map(lambda x: x['name'], _UserSources):
        method = filter(lambda x,u=usource: x['name'] == u, _UserSources) \
            [0]['addmethod']
        getattr(ob, method)(id='User Source')

    for lm in lmeths:
        # Verify that it's a valid method!
        if lm in map(lambda x: x['name'], _LoginMethods):
            method = filter(lambda x,lm=lm: x['name'] == lm, _LoginMethods) \
                [0]['addmethod']
            getattr(ob, method)()
        
    if loginf:
       ob.manage_addDTMLDocument(id='loginForm', file=defLoginForm)

    if logoutf:
       ob.manage_addDTMLDocument(id='logoutForm', file=defLogoutForm)

    if forbidp:
       ob.manage_addDTMLDocument(id='forbiddenPage', file=defForbiddenPage)

    if REQUEST: return self.manage_main(self,REQUEST,update_menu=1)


class LoginManager(Folder):

	"""Thing that manages LoginMethods and UserSources"""

	meta_type = 'Login Manager'
	id = 'acl_users'
	title = 'Login Manager'
	
        __ac_roles__ = ('LoggingIn',)
        
        _super = super
        _nobody = nobody
        
	def __init__(self):
		self.userSourcesList = []
		self.loginMethodsList = []
		
	# Public UserFolder object interface

	def getUser(self,name):
		# Retrieve a user with the given name, or None if no such
		# user exists. Uses the registry of UserSources, after
		# first checking for the superuser.

		if name == self._super.getUserName():
			return self._super

		for source in self.userSourcesList:
			source = source.__of__(self)
			user   = source.getUser(name)
                        if user is not None:
				break
                else:
			# Avoid NameError on user if body of loop doesn't execute
			return None
                                                        
                return user

	def getUserNames(self):
		return []
        
	def getUsers(self):
        	return []
        
	def credentialsChanged(self, user, name, credentials):
		for mname in self.REQUEST.LOGIN_METHODS:
			loginMethod=getattr(self,mname)
			loginMethod.credentialsChanged(self, user, name, credentials)

	def logoutUser(self,name):
		if self.REQUEST.AUTHENTICATED_USER.getUserName()==name:
			for mname in self.REQUEST.LOGIN_METHODS:
				loginMethod=getattr(self,mname)
				loginMethod.logoutCurrentUser(self, name)

	def validate(self, request, auth='', roles=None):
		request.set('LOGIN_METHODS',[])
		if roles is _what_not_even_god_should_do:
			request.response.notFoundError()

		parents=request.get('PARENTS', [])

		if not parents:
			parent=self.aq_parent
		else:	parent=parents[0]

                request['AUTHENTICATED_USER'] = _LoggingInUser
                
		user = None

                if self.loginMethodsList:
	    		for loginMethod in self.loginMethodsList:
    				user = loginMethod.findLogin(self, request, auth, user, roles)
                else:
                        # Can't ever have no login methods;
                        # default to BasicAuth if none selected.
                        user = _DefaultAuth.findLogin(self, request, auth, user, roles)

		if user is not None:
			# We got a user, check him out
			user=getattr(user,'aq_base',user).__of__(self)
			if user.allowed(parent, roles): return user

			# Give the app a chance to tell the user he's logged
			# in, but not allowed
			if hasattr(self,'forbiddenPage'):
			    self.forbiddenPage(self, request, user=user,
			        needroles=roles, userroles=user.getRoles())

		else:	# Fall back to anonymous

			user = self._nobody
			if user.allowed(parent, roles): return user

		# And if anonymous doesn't work (or forbiddenPage
		# doesn't make a fuss), then make 'em log in! 
		# (or fall back to higher level user folders)
		
		if hasattr(self,'loginForm'):
		    self.loginForm(self, request, needroles=roles)

	# LoginManager registration services

	def registerUserSource(self, source):
		srcs = self.userSourcesList
		srcs.append(getattr(source,'aq_base',source))
		self.userSourcesList = srcs
		
	def registerLoginMethod(self, meth):
		meths = self.loginMethodsList
		meths.append(getattr(meth,'aq_base',meth))
		self.loginMethodsList = meths

	def unregisterUserSource(self, source):
	        if hasattr(source, 'aq_base'):
	            source = source.aq_base
	            
		self.userSourcesList = filter(
			lambda x,s=source: x is not s, self.userSourcesList
		)

	def unregisterLoginMethod(self, meth):
	        if hasattr(meth, 'aq_base'):
	            meth = meth.aq_base
	            
		self.loginMethodsList = filter(
			lambda x,m=meth: x is not m, self.loginMethodsList
		)

	def __bobo_traverse__(self, REQUEST, name):
		ob = getattr(self, name, _marker)
		if ob is _marker:
			ob = self.getUser(name)
			if ob is not None:
				return getattr(ob,'aq_base',ob).__of__(self)
			raise 'NotFound'
		return ob			





	
        def all_meta_types(self):
            amt = Folder.all_meta_types
            if callable(amt):
                amt = amt(self)
            return  _UserSources + _LoginMethods + list(amt)
            
	manage_options = (
		{'label':'Contents',        'action':'manage_main'},
		{'label':'User Sources',    'action':'manage_UserSourcesForm'},
		{'label':'Login Methods',   'action':'manage_LoginMethodsForm'},
		{'label':'Properties',      'action':'manage_propertiesForm'},
		{'label':'Security',        'action':'manage_access'},
		{'label':'Undo',            'action':'manage_UndoForm'},
	)

	# Need to add screen(s) where login method and user source priorities
	# can be adjusted; where UseBasicAuth flag can be changed etc.
	# Would also be nice if we had a "manage users" tab which would
	# emulate the standard zopish user folder by delegating to the
	# user sources, or could at least refer the operator to the interfaces
	# of the subordinate user sources

        def LoginMethodsMetaTypes(self):
            global _LoginMethods
            return _LoginMethods
            
        def UserSourcesMetaTypes(self):
            global _UserSources
            return _UserSources 
            
        manage_LoginMethodsForm = HTMLFile(
            'www/LoginMethods', globals(), management_view='Login Methods')

        def manage_loginMethodsFirst(self, ids=[], REQUEST=None):
            """Resort login managers, moving selected ones first"""
            
            l = self.loginMethodsList
            self.loginMethodsList = \
                filter(lambda x,ids=ids: x.id in ids, l) + \
                filter(lambda x,ids=ids: x.id not in ids, l)

            if REQUEST: return self.manage_main(self,REQUEST,update_menu=1)

        manage_UserSourcesForm = HTMLFile(
            'www/UserSources', globals(), management_view='User Sources')

        def manage_userSourcesFirst(self, ids=[], REQUEST=None):
            """Resort user sources, moving selected ones first"""
            
            l = self.userSourcesList
            self.userSourcesList = \
                filter(lambda x,ids=ids: x.id in ids, l) + \
                filter(lambda x,ids=ids: x.id not in ids, l)

            if REQUEST: return self.manage_main(self,REQUEST,update_menu=1)

    	### SimpleItem Protocol ###

	def manage_beforeDelete(self,item,container):
		Folder.manage_beforeDelete(self,item,container)
		if item is self:
			try: del container.__allow_groups__
		        except: pass

	def manage_afterAdd(self,item,container):
		Folder.manage_afterAdd(self,item,container)
		if item is self:
			if hasattr(self, 'aq_base'): self=self.aq_base
			container.__allow_groups__=self
			
	def _setId(self, id):
		if id != self.id:
			raise 'Rename Error', MessageDialog(
				title='Invalid Id',
				message='Cannot change the id of a LoginManager',
				action ='./manage_main',)


def registerLoginMethod(name, addform, addmethod, more=[]):
    global _LoginMethods, LoginManager

    _LoginMethods.append(
        {'name':name, 'action':addform[0], 'addmethod':addmethod[0]})

    meths = [addform, addmethod] + more
    for n, m in meths:
        LoginManager.__dict__[n] = m
    

def registerUserSource(name, addform, addmethod, more=[]):
    global _UserSources, LoginManager

    _UserSources.append(
        {'name':name, 'action':addform[0], 'addmethod':addmethod[0]})

    meths = [addform, addmethod] + more
    for n, m in meths:
        LoginManager.__dict__[n] = m








from LoginMethods import BasicAuth
_DefaultAuth = BasicAuth()  # Convenient singleton used by LoginManager as default login method
