from AccessControl.User import BasicUser
from OFS.SimpleItem import Item
from OFS.Folder import Folder
from LoginManager import registerUserSource
from Globals import HTMLFile, MessageDialog
from Rackmountable import Rackmountable
from string import split
import ZODB.bpthread
import types, sha

# dictionary, indexed by oid of UserSource, of dictionaries, indexed by
# user name, of dictionaries indexed by 'roles', 'domains', ...
_marker = []
_cache = {}
_cache_lock = ZODB.bpthread.allocate_lock()

class LoginUser(BasicUser, Item, Rackmountable):

	"""User that can login by way of a LoginManager.  Delegates actual
	behaviors to the UserSource."""

        isALoginUser = 1
        
        meta_type = 'Login User'
        icon = 'misc_/LoginManager/user'
       
	__propsets__ = ()

	def __init__(self, name):
		self.id=name

	def getUserName(self):
		return self.id

	def getRoles(self):
		return self._v_rack.rolesForUser(self)

	def getDomains(self):
		return self._v_rack.domainsForUser(self)

	def authenticate(self, password, request):
		return self._v_rack.authenticateUser(self, password, request)

	def logout(self):

		"""Log out - can be called as AUTHENTICATED_USER.logout()
		Will only take effect if user has same getUserName() as
		AUTHENTICATED_USER."""

		return self._v_rack.logoutUser(self.getUserName())

	def credentialsChanged(self,credentials):
		self.cacheClear()
		self._v_rack.credentialsChanged(self,self.getUserName(),credentials)

        def cacheClear(self):
		self._v_rack.cacheClear(self.id)

	# propertysheet possibilities:
	# 1. Persistent User, with or without externally managed sheets
	# 2. Non-persistent, external sheets only
	# How do we distinguish, implement each?



class BasicUserSource:
	"""A way of getting user objects, and implementing their innards"""
        isAUserSource = 1


        meta_type = "Basic User Source"
        icon = 'misc_/LoginManager/usersource'

	def getUser(self,name):		pass
	def rolesForUser(self,user):	pass
	def domainsForUser(self,user):	pass
	def authenticateUser(self,user,password,request):
		pass

	# Note: credentialsChanged and logoutUser methods are
	# required, but the default implementation is to
	# acquire these from the LoginManager where the
	# UserSource resides.  The interface for these methods
	# is defined in the LoginManager class.

        def __init__(self, id):
            self.id = id
            self._role_mappings = {}
        
        # ZClass User support
        
        def _User(self, name):
            c = getattr(self, '_zclass', None)
            if c:
                return c._zclass_(name)
            else:
                return LoginUser(name)
            
        manage_classForm = HTMLFile('www/UserClass', globals())

        def manage_class(self, zclass, REQUEST=None):
            """Change class of returned users"""
            self._zclass = None
            for d in self.aq_acquire('_getProductRegistryData')('zclasses'):
                if ("%s/%s" % (d.get('product'),d.get('id'))) == zclass:
                    self._zclass=d['meta_class']
                    break

            if REQUEST is not None:
                return self.manage_main(self, REQUEST, update_menu=1,
                    message='Changed Class.')

        def manage_product_zclass_info(self):
            r=[]
            Z=getattr(self, '_zclass', None)
            Z=getattr(Z, 'aq_self', Z)
            for d in self.aq_acquire('_getProductRegistryData')('zclasses'):
                z=d['meta_class']
                if hasattr(z._zclass_,'_p_deactivate'):
                    # Eek, persistent
                    pass # continue # XXX

                if not hasattr(z._zclass_, 'isALoginUser'):
                    # must be a LoginUser subclass
                    continue

                x={}
                x.update(d)
                x['selected'] = (z is Z) and 'selected' or ''
                del x['meta_class']
                r.append(x)

            return r
        
        # Cache support
        
        def cacheClear(self, name=None):
            global _cache, _cache_lock

            _cache_lock.acquire()
            try:
                if name:
                    try:
                        del _cache[self._p_oid][name]
                    except KeyError:
                        pass
                else:
                    _cache[self._p_oid] = {}
            finally:
                _cache_lock.release()
               
        def cacheSet(self, name, key, value):
            global _cache, _cache_lock

            _cache_lock.acquire()
            try:
                _cache[self._p_oid] = usc = _cache.get(self._p_oid, {})
                usc[name] = uc = usc.get(name, {})
                uc[key] = value
                
            finally:
                _cache_lock.release()
          
        def cacheGet(self, name, key, default=None):
            global _cache #, _cache_lock

            r = default
            # _cache_lock.acquire()
            try:
                try:
                    r = _cache[self._p_oid][name].get(key, default)
                except KeyError:
                    pass
            finally:
                pass # _cache_lock.release()
            return r
            
        def cacheGetAuth(self, name, password):
            """False doesn't mean 'no', it just means we don't know 'yes'"""
            p = self.cacheGet(name, 'pass')
            if p and (sha.sha(password).digest() == p):
                return 1
            return 0
            
        def cacheSetAuth(self, name, password):
            self.cacheSet(name, 'pass', sha.sha(password).digest())

        # Role Mapping Support
        
	def _map_roles(self, roles):
	    # Apply role mappings

            rd = {}

            for r in roles:
                rd[r] = 1

            for r in rd.keys():
                for mr in self._role_mappings.get(r, []):
                    rd[mr] = 1

            return rd.keys()

        manage_mappingsForm = HTMLFile('www/mappings', globals())
        
        def role_mappings_list(self):
            """Get role mappings as from,to pairs"""
            l = []
            for k, v in self._role_mappings.items():
                for x in v:
                    l.append((k, x))
            l.sort()
            return l
                                                                    
        def manage_mappings(self, mapping=[], REQUEST=None):
            """manage role mappings"""
            self._role_mappings = rm = {}
            for m in mapping:
                if m.mfrom and m.mto:
                    if rm.has_key(m.mfrom):
                        rm[m.mfrom].append(m.mto)
                    else:
                        rm[m.mfrom] = [m.mto]

            self.cacheClear()
    
            if REQUEST: return self.manage_main(
                self,REQUEST,update_menu=1,
                manage_tabs_message="Role mappings changed.")

	# Registration

	def manage_beforeDelete(self,item,container):
		self.aq_parent.unregisterUserSource(self)

	def manage_afterAdd(self,item,container):
		container.registerUserSource(self)



class GenericUserSource(BasicUserSource, Folder):
        """A GUF-alike user source"""

        meta_type = "Generic User Source"

	# These four should plug in adapter code from GenericUserFolder
	# to talk to userRoles, userDomains, userAuthenticate, etc.

	def getUser(self,name):
	    user = None
            
            exist = self.cacheGet(name, 'exist', 0)
	    if not exist and hasattr(self, 'userExists'):
                if self.userExists(self, self.REQUEST, username=name) == 1:
                    exist = 1
                    self.cacheSet(name, 'exist', 1)

            if exist:
                user = self._User(name)
                user._setRack(self)

            return user
           
	def rolesForUser(self,user):
	    name = user.id

            data = self.cacheGet(name, 'roles', _marker)
            if data is not _marker:
                return data

            try:
    	        data = self.userRoles(self, self.REQUEST, username=name)
    	        if type(data) == types.StringType:
	            data = split(data)
	        else:
	            data = list(data)
	            if len(data) > 0 and type(data[0]) != types.StringType:
	                if hasattr(data[0], 'role'):
                            data = map(lambda x: x.role, data)
                        elif hasattr(data[0], 'r'):
                            data = map(lambda x: x.r, data)
            except ZeroDivisionError:
	        data = ['Anonymous']
	        # XXX log exception or something
	      
            if 'Anonymous' not in data:
                data.insert(0, 'Anonymous')
                data.sort()
               
            data = self._map_roles(data)
            
            self.cacheSet(name, 'roles', data)
            
            return data

           
	def domainsForUser(self,user):
	    name = user.id

            data = self.cacheGet(name, 'domains', _marker)
            if data is not _marker:
                return data

            try:
    	        data = self.userDomains(self, self.REQUEST, username=name)
    	        if type(data) == types.StringType:
	            data = split(data)
	        else:
	            data = list(data)
	            if len(data) > 0 and type(data[0]) != types.StringType:
	                if hasattr(data[0], 'domain'):
                            data = map(lambda x: x.domain, data)
                        elif hasattr(data[0], 'd'):
                            data = map(lambda x: x.d, data)
            except ZeroDivisionError:
	        data = []
	        # XXX log exception or something
	      
            self.cacheSet(name, 'domains', data)
            
            return data
	   
	def authenticateUser(self,user,password,request):
            if not hasattr(self, 'userAuthenticate'):
                return 0
           
            name = user.id
            
            if self.cacheGetAuth(name, password):
                return 1
                
            ok = self.userAuthenticate(
                self, request, username=name, password=password)
            if ok:
                self.cacheSetAuth(name, password)

            return ok

	# Registration

	def manage_beforeDelete(self,item,container):
		BasicUserSource.manage_beforeDelete.im_func(self,item,container)
		Folder.manage_beforeDelete(self,item,container)

	def manage_afterAdd(self,item,container):
		Folder.manage_afterAdd(self,item,container)
		BasicUserSource.manage_afterAdd.im_func(self,item,container)


        manage_options = tuple(
            [Folder.manage_options[0],
                {'label':'Role Mappings',   'action':'manage_mappingsForm'},
                {'label':'Class',   'action':'manage_classForm'}]
            + list(Folder.manage_options[1:])
        )



manage_addGenericUserSourceForm = HTMLFile('www/addGenericUserSource', globals())

def manage_addGenericUserSource(self, id, title='', REQUEST=None):
    """Add a Generic User Source"""
    ob = GenericUserSource(id)
    ob.id = id
    ob.title = title
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_UserSourcesForm(self, REQUEST, update_menu=1,
            message='Added Generic User Source.')

registerUserSource('Generic User Source',
    addform=('manage_addGenericUserSourceForm', manage_addGenericUserSourceForm),
    addmethod=('manage_addGenericUserSource', manage_addGenericUserSource)
)                        
