class DublinCore:
    """Mix-in class which provides Dublin Core methods"""

    __ac_permissions__=(
        ('View',
         ('Creator','Date', 'Title', 'Description', 'Subject', 'Identifier',
          'Type'),
         ('Owner','Manager','Reviewer')),
        )

    def Title(self):
        "Dublin Core element - resource name"
        return getattr(self, 'title', 'Untitled')

    def Creator(self):
        "Dublin Core element - resource creator"
        if hasattr(self, 'get_local_roles'):
            for user, roles in self.get_local_roles():
                if 'Owner' in roles:
                    return(user)
        return 'No owner'
    Contributor = Creator
    
    def Date(self):
        "Dublin Core element - effective date"
        # Return effective_date (as date object) if set,
        # modification date otherwise
        if getattr(self, 'effective_date', None):
            return self.effective_date
        return self.bobobase_modification_time()

    def Description(self):
        "Dublin Core element - summary"
        # subclasses should override this method
        return getattr(self, 'description', '')

    def Subject(self):
        "Dublin Core element - Topical keywords"
        subject = getattr(self, 'subject', [])
        if type(subject) == type([]): return subject
        if type(subject) == type(()): return list(subject)
        if type(subject) == type(''): return [subject]
        raise ValueError, "subject of unsupported type"

    def Type(self):
        "Dublin Core element - Object type"
        return getattr(self, 'meta_type', 'Unknown type')

    def Identifier(self):
        "Dublin Core element - Object ID"
        return self.id
