from Products.ZCatalog.ZCatalog import ZCatalog
from Products.ZCatalog.Catalog import orify
from SearchIndex import Query
from DateTime import DateTime
import regex

class PortalCatalog(ZCatalog):
    "A catalog which filters searchResults based on user's permission."

    def initialize(self):
        # Reviewing indexes
        self._catalog.addIndex('review_state', 'FieldIndex')
        self._catalog.addIndex('Date', 'FieldIndex')
        self._catalog.addIndex('_allowedRolesAndUsers', 'KeywordIndex')
        # Content indexes
        self._catalog.addIndex('Creator', 'FieldIndex')
        self._catalog.addIndex('Title', 'TextIndex')
        self._catalog.addIndex('Description', 'TextIndex')
        self._catalog.addIndex('SearchableText', 'TextIndex')
        self._catalog.addIndex('Subject', 'KeywordIndex')
        # Catalog meta-data
        self._catalog.addColumn('Subject')
        self._catalog.addColumn('Title')
        self._catalog.addColumn('Description')
        self._catalog.addColumn('review_state')
        self._catalog.addColumn('Creator')
        self._catalog.addColumn('Date')
        self._catalog.addColumn('icon')

    def searchResults(self, REQUEST=None, used=None,
                      query_map={
                          type(regex.compile('')): Query.Regex,
                          type([]): orify, 
                          type(()): orify,
                          type(''): Query.String,
                          }, **kw):
        """
        Wrapper around ZCatalog.searchResults, inserts criteria to ensure
        nobody sees anything they are not allowed to.
        """

        if REQUEST is None and hasattr(self, 'REQUEST'):
            REQUEST = self.REQUEST
        if REQUEST is not None:
            user = REQUEST.AUTHENTICATED_USER
            kw['_allowedRolesAndUsers'] = list(user.getRoles()) + \
                                         ['Anonymous',
                                          'user:'+user.getUserName()]
            if not user.has_permission('Review portal content', self):
                if kw.has_key('Date'):
                    if kw.has_key('Date_usage'):
                        kw['Date'] = min(kw['Date'])
                    kw['Date'] = [kw['Date'], DateTime()]
                    kw['Date_usage'] = 'range:min:max'
                else:
                    kw['Date'] = DateTime()
                    kw['Date_usage'] = 'range:max'
        return apply(ZCatalog.searchResults, (self,REQUEST,used,query_map), kw)

    __call__ = searchResults
