from OFS.Folder import Folder
from Globals import HTMLFile


class PortalFolder(Folder):
    "Implements portal content management"

    # Each module defining a Portal subobject registers with this meta_types
    meta_types = (
        {'name': 'Portal Folder',
         'action': 'manage_addPortalFolderForm',
         'permission': 'Add Folders'},
        )
    
    manage_addPortalFolderForm=HTMLFile('dtml/folderAdd', globals())

    def manage_addPortalFolder(self, id, title='',
                               REQUEST=None):
        """Add a new PortalFolder object with id *id*.
        """
        ob=PortalFolder()
        ob.id=id
        ob.title=title
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.portal_contents(
                self, REQUEST, portal_status_message="Folder added")

    # Portal UI Manager interface

    portal_contents = HTMLFile('dtml/portal_contents',globals())
    
    def portal_cut(self, ids, REQUEST=None):
        "Wrapper around manage_cutObjects, avoids management interface"
        self.manage_cutObjects(ids, REQUEST)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Cut")

    def portal_copy(self, ids, REQUEST=None, RESPONSE=None):
        "Wrapper around manage_copyObjects, avoids management interface"
        self.manage_copyObjects(ids, REQUEST, RESPONSE)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Copied")

    def portal_paste(self, cb_copy_data=None, REQUEST=None):
        "Wrapper around manage_pasteObjects, avoids management interface"
        self.manage_pasteObjects(cb_copy_data, REQUEST)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Pasted")

    portal_renameForm = HTMLFile("dtml/portal_renameForm",globals())

    def portal_rename(self, id, new_id, REQUEST=None):
        """Rename a particular sub-object while avoiding mngmnt interface"""
        try: self._checkId(new_id)
        except: raise CopyError, MessageDialog(
                      title='Invalid Id',
                      message=sys.exc_value,
                      action ='manage_main')
        ob=self._getOb(id)
        if not ob.cb_isMoveable():
            raise CopyError, eNotSupported % id            
        self._verifyObjectPaste(ob, REQUEST)
        try:    ob._notifyOfCopyTo(self, op=1)
        except: raise CopyError, MessageDialog(
                      title='Rename Error',
                      message=sys.exc_value,
                      action ='manage_main')
        self._delObject(id)
        if hasattr(ob, 'aq_base'):
            ob=ob.aq_base
        ob._setId(new_id)
        self._setObject(new_id, ob)
        #ob=ob.__of__(self)            
        #ob._postCopy(self, op=1)
        if REQUEST is not None:
            return self.portal_contents(
                self, REQUEST, portal_status_message="Renamed")
        return None

    def portal_delete(self, ids=[], REQUEST=None):
        "Avoid management interface"
        for id in ids:
            self._delObject(id)
        return self.portal_contents(self, REQUEST,
                                    portal_status_message="Deleted")

manage_addPortalFolder = PortalFolder.manage_addPortalFolder
