import sys, os, string, time, random
from DateTime import DateTime
from DocumentTemplate import HTML
from App.Dialogs import MessageDialog
import OFS.SimpleItem
from urllib import quote, unquote
from Globals import HTMLFile
import Globals

from DublinCore import DublinCore
from Toolbox import Toolbox
import PortalFolder

import MemberFolder
from base64 import encodestring
from urllib import quote


class PortalObjectBase(PortalFolder.PortalFolder, DublinCore):

    meta_type='Portal Site'

    _siteBase=sitePath=''

    __ac_permission__=(
        ('View management screens',
         ('portal_cut', 'portal_copy', 'portal_paste', 'portal_rename',
          'portal_delete')),
        ('Manage portal',
         ('addMemberFolder',), ('Owner',))
        )

    def addMemberFolder(self, REQUEST=None):
        """Add a member folder to a portal site object.

        This is done during initialization."""
        
        if hasattr(self.aq_base, 'acl_users'):
            return MessageDialog(
                   title  ='Item Exists',
                   message='This object already contains a User Folder',
                   action ='%s/manage_main' % REQUEST['URL1'])
        
        dest=self.this()
        ob=self.MembersClass()
        if hasattr(dest, '_setObject'):
            dest._setObject('acl_users', ob)
            dest.__allow_groups__=dest.acl_users
        else:
            dest.__allow_groups__=dest.acl_users=ob
            
        if REQUEST: return dest.manage_main(dest, REQUEST, update_menu=1)

    def addMember(self, name, password, roles, domains):
        self.acl_users._addUser(name, password, password, roles, domains)

    def register(self, username='', full_name='', company='', email='',
                 mail_me=0, REQUEST=None):
        """ """
        register_html=getattr(self, 'join_form')
        if not username or not email:
            return register_html(self, REQUEST,
                   error=('You must enter a value for Name and a valid Email '
                          'Address.'))
    
        user=self.acl_users.getUser(username)
        if user is not None:
            return register_html(self, REQUEST,
                   error=('The login name you selected is already in use. '
                          'Please choose another.'))
        password = REQUEST.get('password', gen_password())
        disapproval = self.password_policy(password)
        if disapproval:
            return register_html(self, REQUEST,
                   error=(disapproval))
        confirm = REQUEST.get('confirm', password)
        if confirm != password:
            return register_html(self, REQUEST,
                   error=('Your password and confirmation did not match. '
                          'Please try again.'))
    
        # Add the member
    
        self.addMember(username, password, ['Member'], [])
    
        user=self.acl_users.getUser(username)
        user.__of__(self.acl_users).setMemberProperties(REQUEST)

        self.register_hook(REQUEST)
        
        # Generate email notification
        # XXX This breaks the division between the PTK and the DemoPortal.
        #     Must move properties from propertysheet to PortalObject.
        if getattr(self, 'validate_email', 0) or mail_me:
            self.registered_notify(self, REQUEST, password=password)
        
        return getattr(self, 'registered')(self, REQUEST)            

    def register_hook(self, *args): pass

    def password_policy(self, candidate):
        """Implement here the password characteristics policy for your site.
    
        Returns an error message on failure, or None if all is ok."""
        if len(candidate) < 5:
            return 'Your password must contain at least 5 characters.'


    def mail_password(self, forgotten_username, REQUEST):
        """Email a forgotten password to a member"""
    
        user = self.acl_users.getUser(forgotten_username)
        if user is None:
            return MessageDialog(
                   title  ='Usename not found',
                   message='The username you entered could not be found.',
                   action ='mail_password_form')
    
        return self.mail_password_template(
            self, REQUEST,
            # full_name=user.full_name,
            email=user.email,
            password=user._getPassword()
            )

    def manage_editSiteBase(self, base='', path='', REQUEST=None):
        """ """
        self._siteBase = base
        self._sitePath = path
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)


    def __bobo_traverse__(self, REQUEST, name=None):
        """ When I site object is traversed, it should reset the
        REQUESTs URL base.  The exception is when the environment
        variable SERVER_URL does not match the new base, in this case, 
        no rewriting should be done (otherwise, your object may become 
        unreachable if your virtual hosts stop working)"""

        if REQUEST['SERVER_URL'] == self._siteBase:
            REQUEST.setSite(self._siteBase)
            # the virtual site's PARENTS should end at the site object
            del REQUEST['PARENTS'][:]

        try: return getattr(self, name)
        except AttributeError: pass
        try: return self[name]
        except KeyError: pass
        method=REQUEST.get('REQUEST_METHOD', 'GET')
        if not method in ('GET', 'POST'):
            return NullResource(self, name, REQUEST).__of__(self)

        REQUEST.RESPONSE.notFoundError("%s\n%s" % (name, method))


    def absolute_url(self, relative=None):
        """ since absolute_url now recurses up the containment
        heirarchy, we want to return an empty string to indicate that
        the recursion should stop here because we are the virtual root 
        for this site object
        """
        # argh, I guess I should aquire REQUEST

        if hasattr(self, 'REQUEST') and \
           self.REQUEST['SERVER_URL'] == self._siteBase:
            if not relative:
                return self._siteBase
            else:
                return ''
        else:
            id=quote(self.id)

            p=getattr(self,'aq_inner', None)
            if p is not None: 
                url=p.aq_parent.absolute_url(relative)
                if url: id=url+'/'+id

            return id

    def changePassword(self, user, password, confirm, domains, REQUEST,
                       redirect=None):
        ' '
        auser=REQUEST['AUTHENTICATED_USER']
        if auser.getUserName() != user:
            raise 'Unauthorized'
        if password != confirm: raise 'Passwords don\'t match'
        auser.changeUser(password, auser.getRoles(), domains)
        token=MemberFolder.digester(user, password).token
        token='%s:%s' % (user, token)
        token=encodestring(token)
        token=quote(token)
        REQUEST.RESPONSE.setCookie('__ac', token, path='/')
        if redirect is not None:
            REQUEST.RESPONSE.redirect(redirect)
        else:
            return self.personalize_form(
                self, REQUEST, portal_status_message="Password changed")

    # Action Provider interface

    def toolbox_actions(self):
        try:
            user=self.REQUEST.AUTHENTICATED_USER
            username=user.getUserName()
        except:
            username='Anonymous User'
            
        if username=='Anonymous User':
            actions = [
                {'name': 'Log in',
                 'url': self.absolute_url() + '/login_form',
                 'permissions': [],
                 'category': 'user'},
                {'name': 'Join',
                 'url': self.absolute_url() + '/join_form',
                 'permissions': [],
                 'category': 'user'},
                ]
        else:
            actions = [
                {'name': 'Preferences',
                 'url': self.absolute_url() + '/personalize_form',
                 'permissions': [],
                 'category': 'user'},
                {'name': 'Log out',
                 'url': self.absolute_url() + '/logout',
                 'permissions' : [],
                 'category': 'user'},
                {'name': 'Undo',
                 'url': 'portal_undoForm',
                 'permissions': [],
                 'category': 'global'},
                {'name': 'Reconfigure portal',
                 'url': self.absolute_url() + '/portal_config',
                 'permissions': ['Manage portal'],
                 'category': 'global'},
                ]
        return actions
    
    # Reviewing Policy interface

    def notify_content(self, changed_object, user):
        # changed_object has just been modified by user.
        # Default policy doesn't care
        changed_object.reindex_object()

    def before_review(self, changed_object, action, review_state, comments,
                      REQUEST):
        # This hook gives the portal a chance to tweak the publishing request 
        return action, review_state, comments

    def review_policy(self, content):
        # The review_state of 'content' has just changed.  Tell the PTK what
        # the new permissions should be.
        if content.review_state == 'private':
            # Revoke 'View' from Anonymous and Reviewer
            return (
                ('Anonymous', (), ('View',)),
                ('Reviewer', (), ('View',)),
                )
        elif content.review_state == 'pending':
            # Give Reviewer 'View', revoke from Anonymous
            return (
                ('Anonymous', (), ('View',)),
                ('Reviewer', ('View',), ()),
                )
        elif content.review_state == 'published':
            # Give Anonymous 'View'
            return (
                ('Anonymous', ('View',), ()),
                )
        raise ValueError, "Unknown state: " + `self.review_state`
        
        
    # Undoing interface

    portal_undoForm = HTMLFile('dtml/portal_undoForm',globals())

    def portal_undo(self, transaction_info, REQUEST=None):
        """Perform undo and present portal interface
        """
        undo=Globals.UndoManager.undo
        for i in transaction_info: undo(i)

        if REQUEST is not None:
            return self.portal_contents(
                self, REQUEST, portal_status_message='Transaction undone')

    def portal_undoable_transactions(self, AUTHENTICATION_PATH=None,
                                     first_transaction=None,
                                     last_transaction=None,
                                     PrincipiaUndoBatchSize=None,
                                     REQUEST=None):
        if REQUEST is None: REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST is not None:
            user = REQUEST.AUTHENTICATED_USER
        else:
            user = None
        transactions = self.undoable_transactions(
            AUTHENTICATION_PATH, first_transaction, last_transaction,
            PrincipiaUndoBatchSize)
        if user and not user.has_role('Manager', self):
            transactions = filter(
                lambda x, user=user:
                    string.split(x['user_name'])[-1] == user.getUserName(),
                transactions
                )
        return transactions

    # Resource locating methods

    def getCatalog(self):
        "Return the portal's catalog"
        # This is potentially flakey because this class does not yet have
        # a way to initialise resources.  We must depend on the application to
        # create 'SiteIndex'.  XXX
        return self.SiteIndex

    def getPortal(self):
        "Return the portal"
        return self

    def getToolbox(self, REQUEST):
        "Return a toolbox"
        return Toolbox().__of__(self)

    def portal_url(self):
        "Return the URL of the portal object, in the client's reckoning."
        # XXX
        return self.absolute_url()


def gen_password(choose=random.choice,
                 chars=string.letters+string.digits, join=string.join):
    result=[]
    for n in range(5):
        result.append(choose(chars))
    return join(result, '')
    
