class ToolboxAware:
    """
    Interface for content which is manageable by the Toolbox.
    
    PortalContent implements this interface, as do each of the
    PortalContent subclasses.
    """
    
    def toolbox_actions(self):
        """
        Report possible toolbox actions
            
        Return: a sequence of dictionaries
            Each dictionary should describe a toolbox action.
            The items of each dictionary are:
                'name' -- string, the name of the action to be displayed
                          in the toolbox.
                          
                'url' -- the url of the action, to be called by the toolbox
                         if the action is chosen.
                         
                'permissions' -- a sequence of permissions any of which
                                 is sufficient to perform the action.
                                 
                'category' -- string 'user' | 'object' | 'global' which
                              describes the category of the action, this
                              is used by the toolbox to display the action.
        
        Permission: 'View management screens'
        """
