import urllib
import OFS, Globals
from PortalFolder import PortalFolder

def registerPortalContent(instance_class,
                          meta_type='',
                          constructors=(),
                          action='',
                          permission="Add portal content",
                          icon=None,
                          ):
    """
    instance_class is the PortalContent-derived class to register
    
    meta_type is it's plain-language name

    contructors is a sequence of constructor functions.  The first is
    placed as a method on PortalFolder.

    action is the relative URL of the add form or wizard

    permission is the name of the permission required to instantiate
    this class

    icon is the name of an image file to use as the document's icon
    """

    meta_type = meta_type or getattr(instance_class, 'meta_type', '')
    
    if constructors:
        initial = constructors[0]
        setattr(PortalFolder, initial.__name__, initial)
    
    PortalFolder.meta_types=PortalFolder.meta_types+(
        {'name': meta_type,
         'action': action,
         'permission': permission},
        )

    if icon:
        path = 'misc_/PTK/%s' % urllib.quote(meta_type)
        instance_class.icon = path
        if not hasattr(OFS.misc_.misc_, 'PTK'):
            OFS.misc_.misc_.PTK = OFS.misc_.Misc_('PTK', {})
        OFS.misc_.misc_.PTK[meta_type] = Globals.ImageFile(icon, globals())
