"""
This module impliments a mickey-mouse UserSource which contains persistent
LoginUsers and a Member mix-in class which provides some hacks to fake the
property sheet interface.

PersistentUserSource and the Member mix-in are meant to attempt to nail
down the external interface.  This should not be taken as an example of a
proper UserSource or LoginUser!!!
"""

from Products.LoginManager.UserSources import BasicUserSource
from OFS.Folder import Folder

from Products.LoginManager.LoginManager import registerUserSource
from Globals import HTMLFile
from Persistence import PersistentMapping

class PersistentUserSource(BasicUserSource, Folder):
    """
    A PersistenUserSource is a UserSource who's LoginUsers are
    contained as persistent Zope objects.
    """

    meta_type = "Persistent User Source"

    manage_options = tuple(
        [Folder.manage_options[0],
         {'label':'Role Mappings',   'action':'manage_mappingsForm'},
         {'label':'Class',   'action':'manage_classForm'}]
         + list(Folder.manage_options[1:])
        )

    def __init__(self, id):
        PersistentUserSource.inheritedAttribute('__init__')(self, id)
        self.__users = PersistentMapping()
        self.__roles = PersistentMapping()
        self.__domains = PersistentMapping()
        self.__ = PersistentMapping()
        
    def getUser(self, id):
        """Fetch a named user"""
        user = (self.__users.has_key(id) and self.__users[id]) or None
        if user: user._setRack(self)
        return user.__of__(self)
    
    def rolesForUser(self,user):
        """Fetch this user's roles"""
        id = user.getUserName()
        return (self.__roles.has_key(id) and self.__roles[id]) or []

    def domainsForUser(self,user):
        """Fetch this user's domains"""
        id = user.getUserName()
        return (self.__domains.has_key(id) and self.__domains[id]) or []

    def authenticateUser(self,user,password,request):
        """Authenticate this user"""
        id = user.getUserName()
        return self.__.has_key(id) and password == self.__[id]

    def changeUser(self, user, password, roles, domains):
        """Change the user 'user'."""
        userid = user.getUserName()
        self.__roles[userid] = roles
        self.__domains[userid] = domains
        self.__[userid] = password

    # I might get burned with this name, but it's very temporary

    def addUser(self, userid, password, roles, domains):
        user = self._User(userid)
        self.__users[userid] = user
        self.__roles[userid] = roles
        self.__domains[userid] = domains
        self.__[userid] = password

class MemberMixin:
    def setMemberProperties(self, REQUEST):
        for ps in self.PropertySheets():
            ps.manage_changeProperties(REQUEST)

    def getHomeUrl(self):
        return self.getPortal().Members[self.getUserName()].absolute_url()

    def toolbox_actions(self):
        """Implements the Toolbox Action Provider interface"""
        return [
            {'name': 'My Stuff',
             'url': self.getHomeUrl()+ '/portal_contents',
             'permissions': [],
             'category': 'user'},
            ]

    def PropertySheets(self):
        """Return all my propery sheets in a list"""
        return self.propertysheets

    def PropertySheet(self, sheet):
        """Return the sheet named by 'sheet'"""
        return getattr(self.propertysheets, sheet)

    def changeUser(self, password, roles, domains):
        """
        Change me!

        These items (pw, roles, doains) will be moved to a designated
        propertysheet in the final implementation.  This method will
        probably hang around after that as a shortcut.
        """
        self._v_rack.changeUser(self, password, roles, domains)
    
manage_addPersistentUserSourceForm = HTMLFile('addPersistentUserSource',
                                              globals())

def manage_addPersistentUserSource(self, id, title='', REQUEST=None):
    """Add a PUS"""
    ob = PersistentUserSource(id)
    ob.id = id
    ob.title = title
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_UserSourcesForm(
            self, REQUEST, update_menu=1,
            message='Added Persistent User Source.')

registerUserSource(
    'Persistent User Source',
    addform = ('manage_addPersistentUserSourceForm',
               manage_addPersistentUserSourceForm),
    addmethod = ('manage_addPersistentUserSource',
                 manage_addPersistentUserSource)
    )
